﻿using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using UnityEngine.Events;

namespace MVRPlugin {
	public class MoveControl : MVRScript {
		protected JSONStorableFloat xMin;
		protected JSONStorableFloat xMax;
		protected JSONStorableFloat yMin;
		protected JSONStorableFloat yMax;
		protected JSONStorableFloat zMin;
		protected JSONStorableFloat zMax;

		protected JSONStorableFloat xPosMin;
		protected JSONStorableFloat xPosMax;
		protected JSONStorableFloat yPosMin;
		protected JSONStorableFloat yPosMax;
		protected JSONStorableFloat zPosMin;
		protected JSONStorableFloat zPosMax;

		protected JSONStorableFloat xPosition;
		protected JSONStorableFloat yPosition;
		protected JSONStorableFloat zPosition;
		protected JSONStorableFloat xIncrement;
		protected JSONStorableFloat yIncrement;
		protected JSONStorableFloat zIncrement;
		protected Vector3 newPos;

		public override void Init() {
			try {
				// put init code in here
				//SuperController.LogMessage("Template Loaded");

				// create custom JSON storable params here if you want them to be stored with scene JSON
				// types are JSONStorableFloat, JSONStorableBool, JSONStorableString, JSONStorableStringChooser
				// JSONStorableColor
				xPosMin = new JSONStorableFloat("X Min Limit ", -100f, -100f, 100f, false, true);
				xPosMax = new JSONStorableFloat("X Max Limit", 100f, -100f, 100f, false, true);
				yPosMin = new JSONStorableFloat("Y Min Limit", -100f, -100f, 100f, false, true);
				yPosMax = new JSONStorableFloat("Y Max Limit", 100f, -100f, 100f, false, true);
				zPosMin = new JSONStorableFloat("Z Min Limit", -100f, -100f, 100f, false, true);
				zPosMax = new JSONStorableFloat("Z Max Limit", 100f, -100f, 100f, false, true);
				RegisterFloat(xPosMin);
				RegisterFloat(xPosMax);
				RegisterFloat(yPosMin);
				RegisterFloat(yPosMax);
				RegisterFloat(zPosMin);
				RegisterFloat(zPosMax);
				CreateSlider(xPosMin);
				CreateSlider(xPosMax,true);
				CreateSlider(yPosMin);
				CreateSlider(yPosMax,true);
				CreateSlider(zPosMin);
				CreateSlider(zPosMax,true);

				xMin = new JSONStorableFloat("X Minimum (loop)", -100f, -100f, 100f, false, true);
				xMax = new JSONStorableFloat("X Maximum (loop)", 100f, -100f, 100f, false, true);
				yMin = new JSONStorableFloat("Y Minimum (loop)", -100f, -100f, 100f, false, true);
				yMax = new JSONStorableFloat("Y Maximum (loop)", 100f, -100f, 100f, false, true);
				zMin = new JSONStorableFloat("Z Minimum (loop)", -100f, -100f, 100f, false, true);
				zMax = new JSONStorableFloat("Z Maximum (loop)", 100f, -100f, 100f, false, true);
				RegisterFloat(xMin);
				RegisterFloat(xMax);
				RegisterFloat(yMin);
				RegisterFloat(yMax);
				RegisterFloat(zMin);
				RegisterFloat(zMax);
				CreateSlider(xMin);
				CreateSlider(xMax,true);
				CreateSlider(yMin);
				CreateSlider(yMax,true);
				CreateSlider(zMin);
				CreateSlider(zMax,true);
				


				xPosition = new JSONStorableFloat("X Position", containingAtom.GetStorableByID("control").transform.position.x,	(float newX)=>{setPosX(newX);}, -5f, 5f, false, true);
				yPosition = new JSONStorableFloat("Y Position", containingAtom.GetStorableByID("control").transform.position.y, (float newY)=>{setPosY(newY);},-5f, 5f, false, true);
				zPosition = new JSONStorableFloat("Z Position", containingAtom.GetStorableByID("control").transform.position.z, (float newZ)=>{setPosZ(newZ);},-5f, 5f, false, true);
				xIncrement = new JSONStorableFloat("X Increment", 0f, -1f, 1f, false, true);
				yIncrement = new JSONStorableFloat("Y Increment", 0f, -1f, 1f, false, true);
				zIncrement = new JSONStorableFloat("Z Increment", 0f, -1f, 1f, false, true);
				RegisterFloat(xPosition);
				RegisterFloat(yPosition);
				RegisterFloat(zPosition);
				RegisterFloat(xIncrement);
				RegisterFloat(yIncrement);
				RegisterFloat(zIncrement);
				CreateSlider(xPosition);
				CreateSlider(yPosition);
				CreateSlider(zPosition);
				CreateSlider(xIncrement, true);
				CreateSlider(yIncrement, true);
				CreateSlider(zIncrement, true);

			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		// Start is called once before Update or FixedUpdate is called and after Init()
		void Start() {
			try {
				// put code in here
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		// Update is called with each rendered frame by Unity
		void Update() {
			try {
				// put code in here

				if(xMin.val>=xMax.val){xMin.val=xMax.val;}
				if(yMin.val>=yMax.val){yMin.val=yMax.val;}
				if(zMin.val>=zMax.val){zMin.val=zMax.val;}
				constantMove();
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		protected void constantMove()
		{
			var newX=containingAtom.GetStorableByID("control").transform.position.x+xIncrement.val/10;
			var newY=containingAtom.GetStorableByID("control").transform.position.y+yIncrement.val/10;
			var newZ=containingAtom.GetStorableByID("control").transform.position.z+zIncrement.val/10;
			var OOB=false;
			FreeControllerV3 FCv3 = containingAtom.GetStorableByID("control") as FreeControllerV3;

			if(newX<xPosMin.val){newX=xPosMin.val;}
			if(newX>xPosMax.val){newX=xPosMax.val;}
			if(newY<yPosMin.val){newY=yPosMin.val;}
			if(newY>yPosMax.val){newY=yPosMax.val;}
			if(newZ<zPosMin.val){newZ=zPosMin.val;}
			if(newZ>zPosMax.val){newZ=zPosMax.val;}

			if(newX<xMin.val){newX=xMax.val;OOB=true;}
			if(newX>xMax.val){newX=xMin.val;OOB=true;}
			if(newY<yMin.val){newY=yMax.val;OOB=true;}
			if(newY>yMax.val){newY=yMin.val;OOB=true;}
			if(newZ<zMin.val){newZ=zMax.val;OOB=true;}
			if(newZ>zMax.val){newZ=zMin.val;OOB=true;}
			newPos=new Vector3(newX,newY,newZ);
			if(OOB)
			{
				//containingAtom.GetStorableByID("control").GetBoolJSONParam("physicsEnabled").val=true;
				//setControllerPositionState("control","On");
			}
			//containingAtom.GetStorableByID("control").transform.position=newPos;
			if(OOB)
			{
				//setControllerPositionState("control","Off");
				//containingAtom.GetStorableByID("control").GetBoolJSONParam("physicsEnabled").val=false;
			}
			if(containingAtom.name.Contains("Person")||FCv3.currentPositionState.ToString()=="On")
			{
				containingAtom.GetStorableByID("control").transform.position=newPos;
			}
			else
			{
				foreach (ForceReceiver fr in containingAtom.forceReceivers)
				{
					if(fr.name=="object")
					{
						fr.transform.position=newPos;
					}
				}
			}
		}

		protected void setPosX(float newX)
		{
			var newY=containingAtom.GetStorableByID("control").transform.position.y;
			var newZ=containingAtom.GetStorableByID("control").transform.position.z;
			newPos=new Vector3(newX,newY,newZ);
			containingAtom.GetStorableByID("control").transform.position=newPos;
		}

		protected void setPosY(float newY)
		{
			var newX=containingAtom.GetStorableByID("control").transform.position.x;
			var newZ=containingAtom.GetStorableByID("control").transform.position.z;
			newPos=new Vector3(newX,newY,newZ);
			containingAtom.GetStorableByID("control").transform.position=newPos;
		}

		protected void setPosZ(float newZ)
		{
			var newY=containingAtom.GetStorableByID("control").transform.position.y;
			var newX=containingAtom.GetStorableByID("control").transform.position.x;
			newPos=new Vector3(newX,newY,newZ);
			containingAtom.GetStorableByID("control").transform.position=newPos;
		}

		protected void setControllerPositionState(string controller, string state)
        {
            FreeControllerV3 FCv3 = containingAtom.GetStorableByID(controller) as FreeControllerV3;
			//Possible states:
			//On, Off, Following, Hold, Lock, ParentLink, PhysicsLink, Comply

			switch (state)
			{
				case "On":
					FCv3.currentPositionState=FreeControllerV3.PositionState.On;
					break;
				case "Off":
					FCv3.currentPositionState=FreeControllerV3.PositionState.Off;
					break;
				case "Following":
					FCv3.currentPositionState=FreeControllerV3.PositionState.Following;
					break;
				case "Hold":
					FCv3.currentPositionState=FreeControllerV3.PositionState.Hold;
					break;
				case "Lock":
					FCv3.currentPositionState=FreeControllerV3.PositionState.Lock;
					break;
				case "ParentLink":
					FCv3.currentPositionState=FreeControllerV3.PositionState.ParentLink;
					break;
				case "PhysicsLink":
					FCv3.currentPositionState=FreeControllerV3.PositionState.PhysicsLink;
					break;
				case "Comply":
					FCv3.currentPositionState=FreeControllerV3.PositionState.Comply;
					break;

				default:
					SuperController.LogError("setControllerPositionState: Unrecognized position status");
					break;
			}
        }

		// FixedUpdate is called with each physics simulation frame by Unity
		void FixedUpdate() {
			try {
				// put code in here
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		// OnDestroy is where you should put any cleanup
		// if you registered objects to supercontroller or atom, you should unregister them here
		void OnDestroy() {
		}

	}
}